//
//  CGOLView.swift
//  ConwaysGameOfLife
//
//  Created by narita on 2018/08/29.
//  Copyright © 2018年 narita. All rights reserved.
//

import Cocoa

class CGOLView: NSView
{
    // MARK: - class methods
    
    override var isFlipped: Bool
    {
        return true
    }
    
    // MARK: - init methods
    
    // MARK: - dealloc
    
    // MARK: - NSCopying, hash, isEqual:
    
    // MARK: - NSCoding methods
    
    // MARK: - restorableState methods
    
    // MARK: - life cycle methods
    
    // MARK: - action methods

    @IBAction func updateModel(_ sender: Any?)
    {
        // viewの広さの更新
        let w: NSInteger = model!.cellsSize.width
        let h: NSInteger = model!.cellsSize.height
        
        let theSize = NSMakeSize(CGFloat(w*10),CGFloat(h*10))
        self.setFrameSize(theSize)
        
        self.setNeedsDisplay(self.bounds)
    }

    
    @IBAction func updateNext(_ sender: Any?)
    {
        self.setNeedsDisplay(self.bounds)
    }
    
    // MARK: - event handling methods
    
    // MARK: - drawing methods
    
    override func draw(_ dirtyRect: NSRect)
    {
        super.draw(dirtyRect)
        
        // 面倒なので、dirtyRectでクリッピングなどはしない。
        
        if let theModel: CGOLModel = self.model
        {
            let w: NSInteger = theModel.cellsSize.width
            let h: NSInteger = theModel.cellsSize.height

            for y in 0 ..< h
            {
                for x in 0 ..< w
                {
                    let thePoint = CGOLPoint(x: x, y: y)
                    let theValue = theModel.alive(at: thePoint)
                    
                    switch theValue
                    {
                        case .alive:
                            NSColor.white.set()
                        case .died:
                            NSColor.gray.set()
                    }
                    
                    let theRect: NSRect = self.modelPoint2Rect(point: thePoint)
                    
                    NSBezierPath.fill(theRect)
                }
            }
        }        
    }

    // MARK: - delegate/datasource methods
    
    // MARK: - accessor methods (in pairs)
    
    var model: CGOLModel?
    {
        didSet
        {
            // viewの広さの更新
            self.updateModel(nil)
            
            // 世代毎に呼ばれるNotificationの登録
            NotificationCenter.default.addObserver(
                self,
                selector: #selector(updateNext(_:)),
                name: .cellsUpdateNotification,
                object: self.model)
            
            // cellSizeが変更された場合にNotificationの登録
            NotificationCenter.default.addObserver(
                self,
                selector: #selector(updateModel(_:)),
                name: .sizeChangeNotification,
                object: self.model)
        }
    }

    // MARK: - Utility methods
    
    private func modelPoint2Rect(point: CGOLPoint) -> CGRect
    {
        // 10 * 10 ptを1cellとする
        let theRect = CGRect(x: point.x * 10,
                             y: point.y * 10,
                             width: 10,
                             height: 10)
        
        return theRect
    }
    
}
